@@ -18,6 +18,9 @@
 #'
 #' @param .call (internal) Used for recording the call to this function.
 #'
+#' @param .skip (internal) If \code{TRUE}, then attempts to set a strategy
+#' that is the same as what is currently in use, will skipped.
+#'
 #' @param .cleanup (internal) Used to stop implicitly started clusters.
 #'
 #' @param .init (internal) Used to initiate workers.
@@ -74,8 +77,8 @@
 #' (HPC) clusters, e.g. LSF, Slurm, TORQUE/PBS, Sun Grid Engine,
 #' and OpenLava.
 #'
-#' To "close" any background workers (e.g. `multisession`), change
-#' the plan to something different; `plan(sequential)` is recommended
+#' To "close" any background workers (e.g. \code{multisession}), change
+#' the plan to something different; \code{plan(sequential)} is recommended
 #' for this.
 #'
 #' @section For package developers:
@@ -96,6 +99,19 @@
 #'   [...]
 #' }
 #'
+#' @section Using plan() in scripts and vignettes:
+#' When writing scripts or vignettes that uses futures, try to place any
+#' call to \code{plan()} as far up (as early on) in the code as possible.  
+#' This will help users to quickly identify where the future plan is set up
+#' and allow them to modify it to their computational resources.
+#' Even better is to leave it to the user to set the \code{plan()} prior to
+#' \code{source()}:ing the script or running the vignette.
+#' If a \file{\link{.future.R}} exists in the current directory and / or in
+#' the user's home directory, it is sourced when the \pkg{future} package is
+#' \emph{loaded}.  Because of this, the \file{.future.R} file provides a
+#' convenient place for users to set the \code{plan()}.
+#'
+#' @importFrom utils capture.output
 #' @export
 plan <- local({
   defaultStrategy <- structure(sequential, call = substitute(plan(sequential)))
@@ -112,7 +128,7 @@ plan <- local({
 
   plan_init <- function() {
     evaluator <- stack[[1L]]
-    init <- attr(evaluator, "init")
+    init <- attr(evaluator, "init", exact = TRUE)
     if (identical(init, TRUE)) {
       debug <- getOption("future.debug", FALSE)
       if (debug) {
@@ -127,7 +143,8 @@ plan <- local({
       stack[[1L]] <<- evaluator
 
       ## Create dummy future to trigger setup (minimum overhead)
-      f <- evaluator(NA, globals = FALSE, lazy = FALSE)
+      f <- evaluator(NA, label = "future-plan-test", 
+                     globals = FALSE, lazy = FALSE)
 
       ## Cleanup, by resolving it
       ## (otherwise the garbage collector would have to do it)
@@ -152,17 +169,72 @@ plan <- local({
   }
 
 
+  equal_strategy_stacks <- function(stack, other) {
+    stop_if_not(is.list(stack), is.list(other))
+    stack <- lapply(stack, FUN = function(s) { attr(s, "call") <- attr(s, "init") <- NULL; s })
+    other <- lapply(other, FUN = function(s) { attr(s, "call") <- attr(s, "init") <- NULL; s })
+
+    if (identical(stack, other)) return(TRUE)
+    if (isTRUE(all.equal(stack, other))) return(TRUE)
+    FALSE
+  }
+
+  plan_set <- function(newStack, skip = TRUE, cleanup = TRUE, init = TRUE) {
+    stop_if_not(!is.null(newStack), is.list(newStack), length(newStack) >= 1L)
+
+    oldStack <- stack
+
+    ## Assign new stack
+    class(newStack) <- unique(c("FutureStrategyList", class(newStack)))
+
+    ## Skip if already set?
+    if (skip && equal_strategy_stacks(newStack, oldStack)) {
+      if (getOption("future.debug", FALSE)) {
+        mdebug(paste0("plan(): Skip setting new future strategy stack because it is the same as the current one:\n", 
+               paste(capture.output(print(newStack)), collapse = "\n"), "\n"))
+      }
+      return(oldStack)
+    }
+
+    if (getOption("future.debug", FALSE)) {
+      mdebug(paste0("plan(): Setting new future strategy stack:\n", 
+             paste(capture.output(print(newStack)), collapse = "\n"), "\n"))
+    }
+    
+    stack <<- newStack
+
+    ## Stop any (implicitly started) clusters?
+    if (cleanup) plan_cleanup()
+
+    ## Initiate future workers?
+    if (init) plan_init()
+
+    ## Sanity checks
+    nbrOfWorkers <- nbrOfWorkers()
+    if (getOption("future.debug", FALSE)) {
+      mdebug(sprintf("plan(): nbrOfWorkers() = %g", nbrOfWorkers))
+    }
+    stop_if_not(is.numeric(nbrOfWorkers), length(nbrOfWorkers) == 1L, 
+                !is.na(nbrOfWorkers), nbrOfWorkers >= 1L)
+
+    invisible(oldStack)
+  }
+
+
   ## Main function
-  function(strategy = NULL, ..., substitute = TRUE, .call = TRUE,
+  function(strategy = NULL, ..., substitute = TRUE, .skip = FALSE, .call = TRUE,
            .cleanup = TRUE, .init = TRUE) {
     if (substitute) strategy <- substitute(strategy)
+    if (is.logical(.skip)) stop_if_not(length(.skip) == 1L, !is.na(.skip))
     if (is.logical(.call)) stop_if_not(length(.call) == 1L, !is.na(.call))
 
     ## Predefined "actions":
     if (is.null(strategy) || identical(strategy, "next")) {
       ## Next future strategy?
       strategy <- stack[[1L]]
-      class(strategy) <- c("FutureStrategy", class(strategy))
+      if (!inherits(strategy, "FutureStrategy")) {
+        class(strategy) <- c("FutureStrategy", class(strategy))
+      }
       return(strategy)
     } else if (identical(strategy, "default")) {
       strategy <- getOption("future.plan", sequential)
@@ -192,15 +264,12 @@ plan <- local({
     targs <- list(...)
 
     ## Set new stack?
-    if (is.list(strategy)) {
-      stop_if_not(is.list(strategy), length(strategy) >= 1L)
+    if (is.function(strategy)) {
+      strategy <- list(strategy)
+    }
 
-      class(strategy) <- unique(c("FutureStrategyList", class(strategy)))
-      stack <<- strategy
-      ## Stop any (implicitly started) clusters?
-      if (.cleanup) plan_cleanup()
-      ## Initiate future workers?
-      if (.init) plan_init()
+    if (is.list(strategy)) {
+      oldStack <- plan_set(strategy, skip = .skip, cleanup = .cleanup, init = .init)
       return(invisible(oldStack[[1L]]))
     }
 
@@ -213,7 +282,7 @@ plan <- local({
         if (is.list(first)) {
           strategies <- first
           res <- plan(strategies, substitute = FALSE,
-	              .cleanup = .cleanup, .init = .init)
+                      .cleanup = .cleanup, .init = .init)
           return(invisible(res))
         }
 
@@ -232,6 +301,7 @@ plan <- local({
             }
           }
           newStack <- strategies
+          stop_if_not(!is.null(newStack), is.list(newStack), length(newStack) >= 1L)
         }
       }
     }
@@ -262,6 +332,7 @@ plan <- local({
 
       ## Setup a new stack of future strategies (with a single one)
       newStack <- list(tstrategy)
+      stop_if_not(!is.null(newStack), is.list(newStack), length(newStack) >= 1L)
     }
 
 
@@ -273,31 +344,16 @@ plan <- local({
       for (kk in seq_along(newStack)) {
         strategy <- newStack[[kk]]
         ## Skip if already has a call attibute
-        if (!is.null(attr(strategy, "call"))) next
+        if (!is.null(attr(strategy, "call", exact = TRUE))) next
         ## Attach call
         attr(strategy, "call") <- call
         newStack[[kk]] <- strategy
       }
+      stop_if_not(!is.null(newStack), is.list(newStack), length(newStack) >= 1L)
     }
 
     ## Set new strategy for futures
-    class(newStack) <- c("FutureStrategyList", class(newStack))
-    stack <<- newStack
-    stop_if_not(is.list(stack), length(stack) >= 1L)
-
-    ## Stop any (implicitly started) clusters?
-    if (.cleanup) plan_cleanup()
-
-    ## Initiate future workers?
-    if (.init) plan_init()
-
-    ## Sanity checks
-    n <- nbrOfWorkers()
-    if (getOption("future.debug", FALSE)) {
-      mdebug(sprintf("plan(): nbrOfWorkers() = %g", n))
-    }
-    stop_if_not(is.numeric(n), length(n) == 1, !is.na(n), n >= 1)
-
+    oldStack <- plan_set(newStack, skip = .skip, cleanup = .cleanup, init = .init)
     invisible(oldStack[[1L]])
   } # function()
 }) # plan()
@@ -311,20 +367,37 @@ supportedStrategies <- function(strategies = c("sequential", "multicore",
 }
 
 
+#' @importFrom utils capture.output
 #' @export
 print.future <- function(x, ...) {
   class <- setdiff(class(x), c("FutureStrategy", "tweaked", "function"))
   s <- sprintf("%s:", class[1])
   specs <- list()
-  args <- deparse(args(x), width.cutoff = 500L)
+  args <- args(x)
+
+  ## Simplify the value on the 'workers' argument?
+  formals <- formals(args)
+  if (!is.atomic(formals$workers) && !is.language(formals$workers)) {
+    bfr <- capture.output(print(formals$workers))
+    if (length(bfr) > 6L) {
+      bfr2 <- capture.output(str(formals$workers))
+      if (length(bfr2) < length(bfr)) bfr <- bfr2
+      if (length(bfr) > 6L) bfr <- c(bfr[1:6], "...")
+    }
+    formals$workers <- paste0("<", paste(bfr, collapse = "; "), ">")
+    formals(args) <- formals
+  }
+
+  args <- deparse(args, width.cutoff = 500L)
   args <- args[-length(args)]
   args <- gsub("(^[ ]+|[ ]+$)", "", args)
   args <- paste(args, collapse = " ")
   specs$args <- args
   specs$tweaked <- inherits(x, "tweaked")
-  specs$call <- paste(deparse(attr(x, "call"), width.cutoff = 500L),
+  specs$call <- paste(deparse(attr(x, "call", exact = TRUE), 
+                              width.cutoff = 500L),
                       collapse="")
-  specs <- sprintf("- %s: %s", names(specs), unlist(specs))
+  specs <- paste0("- ", names(specs), ": ", unlist(specs))
   s <- c(s, specs)
   s <- paste(s, collapse = "\n")
   cat(s, "\n", sep = "")
@@ -335,6 +408,7 @@ print.future <- function(x, ...) {
 print.FutureStrategy <- print.future
 
 
+#' @importFrom utils capture.output
 #' @export
 print.FutureStrategyList <- function(x, ...) {
   s <- "List of future strategies:"
@@ -344,15 +418,32 @@ print.FutureStrategyList <- function(x, ...) {
     class <- setdiff(class(x_kk), c("tweaked", "function"))
     s_kk <- sprintf("%d. %s:", kk, class[1])
     specs <- list()
-    args <- deparse(args(x_kk), width.cutoff = 500L)
+
+    args <- args(x_kk)
+
+    ## Simplify the value on the 'workers' argument?
+    formals <- formals(args)
+    if (!is.atomic(formals$workers) && !is.language(formals$workers)) {
+      bfr <- capture.output(print(formals$workers))
+      if (length(bfr) > 6L) {
+        bfr2 <- capture.output(str(formals$workers))
+        if (length(bfr2) < length(bfr)) bfr <- bfr2
+        if (length(bfr) > 6L) bfr <- c(bfr[1:6], "...")
+      }
+      formals$workers <- paste0("<", paste(bfr, collapse = "; "), ">")
+      formals(args) <- formals
+    }
+
+    args <- deparse(args, width.cutoff = 500L)
     args <- args[-length(args)]
     args <- gsub("(^[ ]+|[ ]+$)", "", args)
     args <- paste(args, collapse = " ")
     specs$args <- args
     specs$tweaked <- inherits(x_kk, "tweaked")
-    specs$call <- paste(deparse(attr(x_kk, "call"), width.cutoff = 500L),
+    specs$call <- paste(deparse(attr(x_kk, "call", exact = TRUE), 
+                                width.cutoff = 500L),
                         collapse = "")
-    specs <- sprintf("   - %s: %s", names(specs), unlist(specs))
+    specs <- paste0("   - ", names(specs), ": ", unlist(specs))
     s <- c(s, s_kk, specs)
   }
 
